module DualReadPortRam (
    input clk,

    input [31:0] DataAddr,
    output [31:0] ReadData,
    input MemWrite,
    input [31:0] WriteData,
    input [3:0] WriteMask,

    input [31:0] InstrAddr,
    output [31:0] Instr
);
    reg [31:0] mem[0:(1024 * 64)];
    
    initial begin
        $readmemh("firmware.hex", mem);
    end

    always @(posedge clk) begin
        if (MemWrite) begin
            $display("mem[%x] <- %x %b", DataAddr, WriteData, WriteMask);

            if(WriteMask[0])
                mem[DataAddr[31:2]][7:0] <= WriteData[7:0];
            if(WriteMask[1])
                mem[DataAddr[31:2]][15:8] <= WriteData[15:8];
            if(WriteMask[2])
                mem[DataAddr[31:2]][23:16] <= WriteData[23:16];
            if(WriteMask[3])
                mem[DataAddr[31:2]][31:24] <= WriteData[31:24];
        end
    end

    assign ReadData = mem[DataAddr[31:2]];
    assign Instr = mem[InstrAddr[31:2]];
endmodule

module RiscVSoc (
    input clk,
    input reset,

    inout [31:0] gpio_pins,
    output tx
);
    wire [31:0] PC;
    wire [31:0] Instr;
    wire MemWrite;
    wire [31:0] DataAddr;
    wire [31:0] WriteData;
    wire [31:0] ReadData;
    wire [3:0] WriteMask;



    RiscV core (
        .clk(clk),
        .reset(reset),
        .PC(PC),
        .Instr(Instr),
        .MemWrite(MemWrite),
        .DataAddr(DataAddr),
        .WriteData(WriteData),
        .ReadData(ReadData),
        .WriteMask(WriteMask)
    );



    Mmio mmio (
        .clk(clk),
        .reset(reset),
        .DataAddr(DataAddr),
        .WriteData(WriteData),
        .WriteMask(WriteMask),
        .MemWrite(MemWrite),
        .ReadData(ReadData),
        .gpio_pins(gpio_pins),
        .tx(tx)
    );

    wire [31:0] ReadDataRam;
    tbuf32 ram_data_buffer (
        .d(ReadDataRam),
        .q(ReadData),
        .en(!DataAddr[31])
    );
    wire MemWriteRam = MemWrite & !DataAddr[31];

    DualReadPortRam ram (
        .clk(clk),
        .DataAddr(DataAddr),
        .ReadData(ReadDataRam),
        .MemWrite(MemWriteRam),
        .WriteData(WriteData),
        .WriteMask(WriteMask),
        .InstrAddr(PC),
        .Instr(Instr)
    );
endmodule