module sim;
	reg clk = 0;
	reg reset = 1;
	always #2 clk = !clk;

    wire tx;
    wire [31:0] gpio_pins;

    RiscVSoc soc (
        .clk(clk),
        .reset(reset),
        .gpio_pins(gpio_pins),
        .tx(tx)
    );

    wire [7:0] uart_out;
    uart_rx rx (clk, reset, uart_out, 32'd234, tx);


	initial begin
        $dumpfile("dump.lxt");
        $dumpvars(0, sim);

		$monitor("gpio: %b", gpio_pins);

        #11 reset = 0;
	
        #400000 $finish;
    end
endmodule

module uart_rx (
    input clk,
    input reset,

    output reg [7:0] byte,

    input [31:0] scaler,

    input rx
);
    localparam STATE_IDLE   = 2'b00;
    localparam STATE_START  = 2'b01;
    localparam STATE_DATA   = 2'b10;
    localparam STATE_STOP   = 2'b11;

    reg [7:0] data = 8'h00;
    reg [2:0] bitpos = 3'h0;
    reg [1:0] state;

    reg[15:0] rx_clkcnt;
    wire rx_clk = (rx_clkcnt == 0);

    always @(posedge clk or posedge reset) begin
        if (reset) begin
            rx_clkcnt <= 0;
        end else begin
            if (rx_clkcnt >= scaler) begin
                rx_clkcnt <= 0;
            end else begin
                rx_clkcnt <= rx_clkcnt + 1;
            end
        end
    end

    always @(posedge clk or posedge reset) begin
        if (reset) begin
            byte <= 8'h00;
            state <= STATE_IDLE;
        end else begin
            case (state)
                STATE_IDLE:
                    begin
                        if (rx == 1'b0) begin
                            state <= STATE_START;
                            bitpos <= 3'h0;
                        end
                    end
                STATE_START:
                    begin
                        if (rx_clk) begin
                            if (rx == 1'b0) begin
                                state <= STATE_DATA;
                            end else begin
                                state <= STATE_IDLE;
                            end
                        end
                    end
                STATE_DATA:
                    begin
                        if (rx_clk) begin
                            if (bitpos == 3'h7)
                                state <= STATE_STOP;
                            else
                                bitpos <= bitpos + 3'h1;
                            data[bitpos] <= rx;
                        end
                    end
                STATE_STOP:
                    begin
                        if (rx_clk) begin
                            if (rx == 1'b1) begin
                                byte <= data;
                                $display("%c", byte);
                                state <= STATE_IDLE;
                            end
                        end
                    end
            endcase
        end
    end
endmodule
